@@ -1,4 +1,4 @@
-static char rcsid[] = "$Id: approve.c,v 1.135 2000/09/13 15:17:26 will Exp $";
+static const char rcsid[] = "$Id: approve.c,v 1.142 2004/04/30 17:00:58 will Exp $";
 
 /* The code should compile with either ANSI C or K&R compilers. */
 
@@ -118,11 +118,12 @@ int verbosity;		/* HELP_BASIC, HELP_FULL, or HELP_FACTS */
 	}
 
 	/* Initialize localinfo */
-	if (handle_option(NULL, NULL, 0) == -1)
+	if (handle_option(NULL, NULL, 0) == -1) {
 	    if (check_syntax)
 		continue;
 	    else
 		return NULL;
+	}
 
 	if (check_syntax) {
 	    parseline(0, 1, buf, usrcmd);
@@ -584,16 +585,25 @@ char *cond_or_opt_wd;	/* First word to process; rest we get with strqtokS */
 	    globalinfo.userbefore.next) {
 	/* We have user fields to match */
 
-	/* Default for root is to allow; default for all others is disallow */
-	matches.user = (userinfo.caller.pw_uid == 0) ? 1 : 0;
+	matches.user = -1;	/* no yes/no has yet been made for this user */
 
+	/* Start with the global pats that get put after the per-line pats */
 	match_ugh_user(&globalinfo.userafter, 1);
-	if (!matches.user) {
+	if (matches.user == -1) {
+	    /* no yes or no yet for this user; try the per-line pats */
 	    match_ugh_user(&localinfo.userpats, 0);
-	    if (!matches.user) {
+	    if (matches.user == -1) {
+		/* no yes or no yet; try the global before-line pats */
 		match_ugh_user(&globalinfo.userbefore, 1);
 	    }
 	}
+	if (matches.user == -1) {
+	    /* No yes or no for this user; use a default.
+	     * Default for root is to allow;
+	     * default for all others is disallow
+	     */
+	    matches.user = (userinfo.caller.pw_uid == 0) ? 1 : 0;
+	}
     } else {
 	return Error(0, 0,
 	"%t\n\tNo user patterns given on control line or in :global line.\n");
@@ -651,20 +661,21 @@ char *usrcmd;
 "  %s command [args...]   ......   executes command [args].",
 "",
 "  %s   ........................   basic listing of permitted cmds.",
-"  %s [-H | -f] [MasqueradeOpts]   list ok cmds: -H=verbose,-f=just-facts.",
-"  %s [-H | -f] [MasqueradeOpts] cmd  ...give help only for `cmd'.",
+"  %s [-H | -f] [-S] [MasqOpts]   list ok cmds: -H=verbose,-f=just-facts.",
+"  %s [-H | -f] [-S] [MasqOpts] cmd  ...give help only for `cmd'.",
 "  %s -h   .....................   print this usage info, then exits.",
 "",
 "  %s -b   .....................   lists builtin variables, then exits.",
 "  %s -c [superfile]   .........   just check syntax of the superfile.",
 "  %s -V   .....................   print version number, then exits.",
-"  %s [-d | -D] [MasqueradeOpts] cmd [args]   ...debug mode.",
+"  %s [-d | -D] [-S] [MasqOpts] cmd [args]   ...debug mode.",
 "",
 "Notes on options:",
 "    -d   Basic debug: show what would be done, but don't execute anything.",
 "    -D   Same as -d, plus tell more about variables, plus other details.",
 "    -f   (just the facts, ma'm): lists your cmds as: Cmd FullPath [args]",
 "	      Useful for scripts that want a list of what you may execute.",
+"    -S -- force stdin to be used for reading passwords.",
 "",
 "MasqueradeOptions:",
 "    ** No command is ever executed when any MasqueradeOption is used.**",
@@ -837,15 +848,25 @@ int verbosity;
 	    }
 	}
     }
-    if (*localinfo.user ||
-	    *localinfo.group ||
-	    *localinfo.u_g ||
-	    localinfo.fdlist ||
-	    *localinfo.owner ||
-	    localinfo.env) {
+    if ( *localinfo.user || *localinfo.euser ||
+	    *localinfo.group || *localinfo.egroup || *localinfo.u_g ||
+	    localinfo.fdlist || *localinfo.owner || localinfo.env) {
 	(void) fputs("\tExecutes with:", stdout);
-	if (*localinfo.user) (void) printf(" uid=%s", localinfo.user);
-	if (*localinfo.group) (void) printf(" gid=%s", localinfo.group);
+	if (*localinfo.euser) {
+	    	(void) printf(" euid=%s", localinfo.euser);
+	    if (*localinfo.user)
+	    	(void) printf(" ruid=%s", localinfo.user);
+	} else if (*localinfo.user) {
+	    (void) printf("  uid=%s", localinfo.user);
+	}
+	if (*localinfo.egroup) {
+	    	(void) printf(" egid=%s", localinfo.egroup);
+	    if (*localinfo.group) {
+	    	(void) printf("  gid=%s", localinfo.group);
+	    }
+	} else if (*localinfo.group) {
+	    (void) printf(" gid=%s", localinfo.group);
+	}
 	if (*localinfo.u_g) (void) printf(" u+g=%s", localinfo.u_g);
 	if (localinfo.env) {
 	    for (p=localinfo.env; *p; p++)
