@@ -1,4 +1,4 @@
-static char rcsid[] = "$Id: checks.c,v 1.147 2000/12/15 17:12:54 will Exp $";
+static const char rcsid[] = "$Id: checks.c,v 1.163 2004/04/30 17:00:58 will Exp will $";
 /* The code should compile with either ANSI C or K&R compilers. */
 
 /*
@@ -20,12 +20,13 @@ static char rcsid[] = "$Id: checks.c,v 1.147 2000/12/15 17:12:54 will Exp $";
 #define netgrp_h_compare(p, h) 0
 #endif
 
+int get_setting P__ (());
+
 
 #ifdef _HPUX_SOURCE
 
 #ifdef HAVE_ISCOMSEC
     char *bigcrypt();
-    char *getpasswd();
 #else
     /* If we don't have iscomsec() on this HP system, fake it -- otherwise
      * we'll have to #ifdef the use of iscomsec(), and it's cleaner code
@@ -458,56 +459,187 @@ int
 set_u_g()
 {
     /* Return 0 on success, -1 on failure */
-    struct passwd *uid_pw;
     SETGRENT_T setgrent();
     void endgrent();
-    int i, found_gid;
+    int i, j, k = -1;
+    int found_gid = -1, found_egid = -1;
+    struct passwd *pw = NULL;
+    int use_setreuid;	/* true iff we need to use setreuid() */
+    int use_setregid;	/* true iff we need to use setregid() */
+    uid_t uid_pw_uid = -1, euid_pw_uid = -1;
+    gid_t u_g_pw_gid = -1;
+    char *uid_pw_name = NULL, *euid_pw_name = NULL;
 
-    userinfo.new_uid = userinfo.caller.pw_uid;
-    userinfo.new_gid = userinfo.caller.pw_gid;
+    /*
+     * Start by looking up user id's and group id's.  We'll assign
+     * actual values after we've looked it all up.
+     */
 
-    /* Check gid=xxx */
+    /* First, get the group id's specified by {e,}gid=xxx */
     if (*localinfo.group) {
 	found_gid = findgid(1, localinfo.group);
 
-	if (found_gid == -1)
+	if (found_gid == -1) {
 	    return Error(0, 0,
-		"%t\n\tCan't set gid: no such group as `%s' in group file.\n",
+	    "%t\n\tCan't set gid: no such group as `%s' in group file.\n",
 		localinfo.group);
-	userinfo.new_gid = found_gid;
+	}
+    }
+    if (*localinfo.egroup) {
+	found_egid = findgid(1, localinfo.egroup);
+
+	if (found_egid == -1) {
+	    return Error(0, 0,
+	    "%t\n\tCan't set egid: no such group as `%s' in group file.\n",
+		localinfo.egroup);
+	}
     }
 
-    /* Check uid=xxx u+g=yyy */
-    if (*localinfo.user || *localinfo.u_g) {
-	if (*localinfo.u_g)
-	    strcpy(localinfo.user, localinfo.u_g);
-	
-	uid_pw = getpwentry(1, localinfo.user);
-	if (!uid_pw)
+    /* Second, get the password entries specified by {e,}uid=xxx, u+g=xxx */
+    if (*localinfo.u_g && !*localinfo.user) {
+	/* u+g=zzz was used, but uid=xxx was not.  So, set user from u+g. */
+	strcpy(localinfo.user, localinfo.u_g);
+    }
+
+    if (*localinfo.user) {
+	pw = getpwentry(1, localinfo.user);
+	if (!pw)
 	    return -1;
-	userinfo.new_uid = uid_pw->pw_uid;
-	if (*localinfo.u_g) {
-	    userinfo.new_gid = uid_pw->pw_gid;
+	uid_pw_uid  = pw->pw_uid;
+	uid_pw_name = strdup(pw->pw_name);
+	if (!uid_pw_name) 
+		return Error(0, 0,
+		    "%t\n\tfailed to malloc space for uid_pw_name=<%s>\n", pw->pw_name);
+	
+    }
+    if (*localinfo.euser) {
+	pw = getpwentry(1, localinfo.euser);
+	if (!pw)
+	    return -1;
+	euid_pw_uid  = pw->pw_uid;
+	euid_pw_name = strdup(pw->pw_name);
+	if (!euid_pw_name)
+		return Error(0, 0,
+		    "%t\n\tfailed to malloc space for euid_pw_name=<%s>\n", pw->pw_name);
+    }
+    if (*localinfo.u_g) {
+	pw = getpwentry(1, localinfo.u_g);
+	if (!pw)
+	    return -1;
+	u_g_pw_gid  = pw->pw_gid;
+    }
+
+    pw = NULL;
+    
+    if (*localinfo.group && *localinfo.u_g) {
+	return Error(0, 0,
+		"%t\n\tCan't mix options gid=xxx and u+g=yyy in one entry.");
+    }
+
+    /*
+     * OK, now we have all the uid/gid info.  Select uid's and gid's,
+     * and select the appropriate set of functions to do the assignments.
+     */
+
+    /* UID defaults: */
+    userinfo.new_uid = userinfo.caller.pw_uid;
+    use_setreuid = 0;		/* don't have to use setreuid() */
+
+    if (*localinfo.euser) {
+	/* euid=xxx was used */
+	userinfo.new_euid = euid_pw_uid;
+	if (strcmp(euid_pw_name, localinfo.euser) != 0) {
+	    /* localinfo.euser must be numeric; convert it to string */
+	    strcpy(localinfo.euser, euid_pw_name);
 	}
-	if (strcmp(uid_pw->pw_name, localinfo.user) != 0) {
+	use_setreuid = 1;	/* have to use setreuid() */
+    }
+
+    if (*localinfo.user) {
+	/* uid=xxx or u+g=xxx was used */
+	userinfo.new_uid = uid_pw_uid;
+	if (strcmp(uid_pw_name, localinfo.user) != 0) {
 	    /* localinfo.user must be numeric; convert it to string */
-	    strcpy(localinfo.user, uid_pw->pw_name);
+	    strcpy(localinfo.user, uid_pw_name);
 	}
     }
 
+    /* GID defaults: */
+    userinfo.new_gid = userinfo.caller.pw_gid;
+    use_setregid = 0;		/* don't have to use setreuid() */
+
+    if (*localinfo.egroup) {
+	/* egid=xxx was used */
+	userinfo.new_egid = found_egid;
+	use_setregid = 1;	/* have to use setreuid() */
+    }
+
+    if (*localinfo.group) {
+	/* gid=xxx was used */
+	userinfo.new_gid = found_gid;
+    } else if (*localinfo.u_g) {
+	userinfo.new_gid = u_g_pw_gid;
+    }
+
     /* Set supplementary groups */
     if (set_suppl_groups() == -1)
 	return -1;
 
 
     /* Now set uid & gid */
-    if (*localinfo.group || *localinfo.u_g)
-	if (setgid(userinfo.new_gid) == -1)
+    if (use_setregid) {
+#ifdef HAVE_SETREGID
+	if ((i=setregid(userinfo.new_gid, userinfo.new_egid)) == -1) {
+	    return Error(1, 0, "setregid(gid=%d, egid=%d) failed: ",
+		    	userinfo.new_gid, userinfo.new_egid);
+	} else if ((j=getgid()) != userinfo.new_gid ||
+				    (k=getegid()) != userinfo.new_egid) {
+	    return Error(0, 0,
+	"setregid(gid=%d,egid=%d) returned 0, but getgid()=%d,getegid()=%d!",
+		userinfo.new_gid, userinfo.new_egid, j, k);
+	}
+#else
+	return Error(0, 0, "Can't use egid=xxx option, because this host doesn't offer the setregid() function");
+#endif
+    } else if (*localinfo.group || *localinfo.u_g) {
+	if ((i=setgid(userinfo.new_gid)) == -1) {
 	    return Error(1, 0, "setgid(gid=%d) failed: ", userinfo.new_gid);
-    if (*localinfo.user || *localinfo.u_g)
-	if (setuid(userinfo.new_uid) == -1)
-	    return Error(1, 0, "setuid(uid=%d) failed: ", userinfo.new_uid);
+	} else if ((j=getgid()) != userinfo.new_gid) {
+	    return Error(0, 0,
+		"setgid(gid=%d) returned %d, but getgid() returned %d!",
+		userinfo.new_gid, i, j);
+	}
+    }
 
+
+    if (use_setreuid) {
+#ifdef HAVE_SETREUID
+	if ((i=setreuid(userinfo.new_uid, userinfo.new_euid)) == -1) {
+	    return Error(1, 0, "setreuid(uid=%d, euid=%d) failed: ",
+		    	userinfo.new_uid, userinfo.new_euid);
+	} else if ((j=getuid()) != userinfo.new_uid ||
+				    (k=geteuid()) != userinfo.new_euid) {
+	    return Error(1, 0,
+	"setreuid(uid=%d,euid=%d) returned 0, but getuid()=%d,geteuid()=%d!",
+		userinfo.new_uid, userinfo.new_euid, j, k);
+	}
+#else
+	return Error(1, 0, "Can't use euid=xxx option, because this host doesn't offer the setreuid() function");
+#endif
+    } else if (*localinfo.user || *localinfo.u_g) {
+	if ((i=setuid(userinfo.new_uid)) == -1) {
+	    return Error(1, 0, "setuid(uid=%d) failed: ", userinfo.new_uid);
+	} else if ((j=getuid()) != userinfo.new_uid) {
+	    return Error(1, 0,
+		"setuid(uid=%d) returned %d, but getuid() returned %d!",
+		userinfo.new_uid, i, j);
+	}
+    }
+
+    if (uid_pw_name) 
+	    free(uid_pw_name);
+    if (euid_pw_name)
+	    free(euid_pw_name);
     return 0;
 }
 
@@ -518,6 +650,8 @@ set_u_g()
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
 #ifdef SUNOS5 
+#define HAVE_G_E_PW
+
 int
 get_encrypted_pw()
 {
@@ -528,13 +662,12 @@ get_encrypted_pw()
 	    "Failed to obtain shadow password entry for user %s: ",
 	    userinfo.caller.pw_name);
     strcpy(userinfo.encr, caller_pw->sp_pwdp);
-    strncpy(userinfo.salt, caller_pw->sp_pwdp, 2);
-    userinfo.salt[2] = '\0';
-    return 0;
+    return get_setting();
 }
+#endif
 
-#else
 #ifdef _HPUX_SOURCE
+#define HAVE_G_E_PW
 
 int
 get_encrypted_pw()
@@ -548,8 +681,6 @@ get_encrypted_pw()
 		"Failed to obtain Trusted Computing Base entry for user %s: ",
 		userinfo.caller);
 	strcpy(userinfo.encr, caller_pw->ufld.fd_encrypt);
-	strncpy(userinfo.salt, caller_pw->ufld.fd_encrypt, 2);
-	userinfo.salt[2] = '\0';
 
     } else {
 	struct passwd *caller_pw;
@@ -567,21 +698,19 @@ get_encrypted_pw()
 		    "Failed to obtain shadow password entry for user %s: ",
 		    userinfo.caller);
 	    strcpy(userinfo.encr, caller_pw->pw_passwd);
-	    strncpy(userinfo.salt, caller_pw->pw_passwd, 2);
-	    userinfo.salt[2] = '\0';
 	} else {
 
 	    /* Fall through to regular password file lookup. */
 	    strcpy(userinfo.encr, userinfo.caller.pw_passwd);
-	    strncpy(userinfo.salt, userinfo.caller.pw_passwd, 2);
-	    userinfo.salt[2] = '\0';
 	}
     }
-    return 0;
+    return get_setting();
 }
+#endif
 
-#else
 #ifdef SCO
+#define HAVE_G_E_PW
+
 int
 get_encrypted_pw()
 {
@@ -602,20 +731,18 @@ get_encrypted_pw()
 	    "Failed to obtain shadow password entry for user %s: ",
 	    userinfo.caller.pw_name);
 	strcpy(userinfo.encr, caller_ps->sp_pwdp);
-	strncpy(userinfo.salt, caller_ps->sp_pwdp, 2);
-	userinfo.salt[2] = '\0';
 
     } else {
 	/* Fall through to regular password file lookup. */
 	strcpy(userinfo.encr, caller_pw->pw_passwd);
-	strncpy(userinfo.salt, caller_pw->pw_passwd, 2);
-	userinfo.salt[2] = '\0';
     }
-    return 0;
+    return get_setting();
 }
+#endif
 
-#else
 #ifdef Digital_UNIX
+#define HAVE_G_E_PW
+
 int
 get_encrypted_pw()
 {
@@ -636,20 +763,17 @@ get_encrypted_pw()
 	    "Failed to obtain shadow password entry for user %s: ",
 	    userinfo.caller.pw_name);
 	strcpy(userinfo.encr, caller_prpass->ufld.fd_encrypt);
-	strncpy(userinfo.salt, caller_prpass->ufld.fd_encrypt, 2);
-	userinfo.salt[2] = '\0';
 
     } else {
 	/* Fall through to regular password file lookup. */
 	strcpy(userinfo.encr, caller_pw->pw_passwd);
-	strncpy(userinfo.salt, caller_pw->pw_passwd, 2);
-	userinfo.salt[2] = '\0';
     }
-    return 0;
+    return get_setting();
 }
+#endif
 
-#else
 #ifdef __linux__
+#define HAVE_G_E_PW
 
 int
 get_encrypted_pw()
@@ -658,22 +782,72 @@ get_encrypted_pw()
     static struct stat st;
     if (stat("/etc/shadow", &st) == 0) {
 	struct spwd *spwd = 0L;
-	if (!(spwd = getspnam(userinfo.caller.pw_name)))
-	    return Error(1, 0,
-		"Failed to obtain shadow password entry for user %s: ",
-		userinfo.caller.pw_name);
-	strcpy(userinfo.encr, spwd->sp_pwdp);
-	strncpy(userinfo.salt, spwd->sp_pwdp, 2);
-	userinfo.salt[2] = '\0';
+	if (!(spwd = getspnam(userinfo.caller.pw_name))) {
+	    /* Gordon Lack notes: Linux doesn't use the shadow file for NIS
+	     * passwords, so if you get ENOENT or no error, then try the
+	     * local password file.
+	     */
+	    if (errno == ENOENT || errno == 0) {
+		strcpy(userinfo.encr, userinfo.caller.pw_passwd);
+	    } else {
+		return Error(1, 0,
+		    "Failed to obtain shadow password entry for user %s: ",
+		    userinfo.caller.pw_name);
+	    }
+        } else {
+	    /* got shadow entry */
+	    strcpy(userinfo.encr, spwd->sp_pwdp);
+	}
     } else {
+	/* Fall through to regular password file lookup. */
 	strcpy(userinfo.encr, userinfo.caller.pw_passwd);
-	strncpy(userinfo.salt, userinfo.caller.pw_passwd, 2);
-	userinfo.salt[2] = '\0';
     }
-    return 0;
-}
 
-#else
+    return get_setting();
+}
+#endif
+
+#ifdef __FreeBSD__
+#define HAVE_G_E_PW
+
+int
+get_encrypted_pw()
+{
+    /* getpwnam() and getpwuid() will have read the shadow passwd
+     * if our effective uid == root; no need to use getspnam() or the like.
+     * But if the encrypted pw is "*", then we didn't get the real pw.
+     */
+    strcpy(userinfo.encr, userinfo.caller.pw_passwd);
+    if (strcmp(userinfo.encr, "*") == 0) {
+	return Error(0, 0, "Can't read encrypted password for user %s",
+			userinfo.caller.pw_name);
+    }
+
+    return get_setting();
+}
+#endif
+
+#ifdef __OpenBSD__
+#define HAVE_G_E_PW
+
+int
+get_encrypted_pw()
+{
+    /* getpwnam() and getpwuid() will have read the shadow passwd
+     * if our effective uid == root; no need to use getspnam() or the like.
+     * But if the encrypted pw is "*", then we didn't get the real pw.
+     */
+    strcpy(userinfo.encr, userinfo.caller.pw_passwd);
+    if (strcmp(userinfo.encr, "*") == 0) {
+	return Error(0, 0, "Can't read encrypted password for user %s",
+			userinfo.caller.pw_name);
+    }
+
+    return get_setting();
+}
+#endif
+
+#ifndef HAVE_G_E_PW
 
 int
 get_encrypted_pw()
@@ -682,15 +856,52 @@ get_encrypted_pw()
 
     strcpy(userinfo.encr, userinfo.caller.pw_passwd);
     strncpy(userinfo.salt, userinfo.caller.pw_passwd, 2);
-    userinfo.salt[2] = '\0';
+    return get_setting();
+}
+#endif
+
+/*
+ * Read the userinfo.encr (encrypted passwd field) and pull out the data
+ * needed for the "salt" or "setting" argument to crypt().
+ * This version implements the following rule:
+ * o  if encr begins with "$", the entire encr field should be used
+ * 	(crypt() will be responsible for extracting the data it needs);
+ * o  otherwise, if encr begins with "_", use the first 9 chars
+ * 	(crypt() will discard the leading "_" and use the next 8 characters);
+ * o  otherwise, the salt is the first two chars.
+ *
+ * Returns:
+ * -1 (plus Error()) on failure
+ * 0 on success.
+ */
+int
+get_setting()
+{
+    if (sizeof(userinfo.salt) <= strlen(userinfo.encr)) {
+	return Error(0, 0, "Compilation error: the userinfo.salt field is too short (%d chars) to hold a copy of the userinfo.encr field (%d chars) for user %s",
+			sizeof(userinfo.salt), strlen(userinfo.encr),
+			userinfo.caller.pw_name);
+    }
+
+    if (userinfo.encr[0] == '$') {
+	strcpy(userinfo.salt, userinfo.encr);
+
+    } else if (userinfo.encr[0] == '_') {
+	/* crypt() _might_ parse the encr field to pull off the necessary
+	 * salt, but we don't make that assumption.
+	 */
+	strncpy(userinfo.salt, userinfo.encr, 9);
+	userinfo.salt[9] = '\0';
+
+    } else {
+	/* Salt is first two chars. */
+	strncpy(userinfo.salt, userinfo.encr, 2);
+	userinfo.salt[2] = '\0';
+    }
+
     return 0;
 }
 
-#endif
-#endif
-#endif
-#endif
-#endif
 
 
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
@@ -709,6 +920,7 @@ auth_name()
 						localinfo.authinfo.method);
     }
     /* NOTREACHED */
+    return "Unknown!"; /* to make gcc -Wall happy... */
 }
 
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
@@ -897,22 +1109,32 @@ char *salt;
 {
     char *crypt();
     char *getpass();
+    char buf[300];
+    int n;
+
+    n = s_getpass(prompt, use_stdin, buf, sizeof(buf));
+    if (n < 0) {
+	Error(0, 1, "Failed to get password.\n");
+    } else if (n >= sizeof(buf)) {
+	Error(0, 1, "Buffer too small (%d chars) to hold input password!\n",
+								sizeof(buf)-1);
+    }
 
 #ifdef _HPUX_SOURCE
 
     if (iscomsec()) {
 	#if (HPUX_MAJOR == 10)
-	    return bigcrypt(getpasswd(prompt), salt);
+	    return bigcrypt(buf, salt);
 	#else
-	    return crypt(getpass(prompt), salt);
+	    return crypt(buf, salt);
 	#endif
     } else {
-	return crypt(getpass(prompt), salt);
+	return crypt(buf, salt);
     }
 
 #else
 
-    return crypt(getpass(prompt), salt);
+    return crypt(buf, salt);
 
 #endif
 }
