@@ -1,4 +1,4 @@
-static char rcsid[] = "$Id: options.c,v 1.105 2000/09/15 21:50:53 will Exp $";
+static const char rcsid[] = "$Id: options.c,v 1.118 2004/04/30 17:00:58 will Exp $";
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 /* Process a global xxx=yyy option */
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
@@ -22,7 +22,9 @@ Option opts[] = {
 	{ "cd",			GLOBAL|LOCAL,	option_cd },
 	{ "checkvar",		LIST|LOCAL,	option_checkvar },
 	{ "die",		LOCAL,		option_die },
+	{ "egid",		LOCAL,		option_egid },
 	{ "env",		LIST|GLOBAL|LOCAL,	option_env },
+	{ "euid",		LOCAL,		option_euid },
 	{ "fd",			LIST|LOCAL,	option_fd },
 	{ "gethostbyname",	GLOBAL,		option_gethostbyname },
 	{ "gid",		LOCAL,		option_gid },
@@ -274,7 +276,9 @@ option_local_clear_settings()
     if (localinfo.argv0)
 	free(localinfo.argv0);
     localinfo.argv0 = NULL;
-    *localinfo.user = *localinfo.group = *localinfo.u_g = '\0';
+    *localinfo.euser = *localinfo.user = '\0';
+    *localinfo.egroup = *localinfo.group = '\0';
+    *localinfo.u_g = '\0';
     if (localinfo.env)
 	blkfree(localinfo.env);
     if (globalinfo.env) {
@@ -444,8 +448,6 @@ char *word;
 char *s;
 int isglobal;
 {
-    int method;
-
     if (debug)
 	fprintf(stderr, "\toption:authuser=%s (%s)\n",
 				s, isglobal ? "global" : "local");
@@ -596,6 +598,34 @@ int isglobal;
     return 0;
 }
 
+static int
+option_egid(word, s, isglobal)
+char *word;
+char *s;
+int isglobal;
+{
+    if (debug)
+	fprintf(stderr, "\toption:egid=%s (%s)\n",
+				s, isglobal ? "global" : "local");
+
+    stringcopy(localinfo.egroup, s, sizeof(localinfo.egroup));
+    return 0;
+}
+
+static int
+option_euid(word, s, isglobal)
+char *word;
+char *s;
+int isglobal;
+{
+    if (debug)
+	fprintf(stderr, "\toption:euid=%s (%s)\n",
+				s, isglobal ? "global" : "local");
+
+    stringcopy(localinfo.euser, s, sizeof(localinfo.euser));
+    return 0;
+}
+
 static int
 option_env(word, s, isglobal)
 char *word;
@@ -1034,7 +1064,7 @@ char *word;
 char *s;
 int isglobal;
 {
-    int method, required;
+    int required;
 
     if (debug)
 	fprintf(stderr, "\toption:password=%s (%s)\n",
@@ -1073,6 +1103,7 @@ char *word;
 char *s;
 int isglobal;
 {
+    char *p;
     if (debug)
 	fprintf(stderr, "\toption:patterns=%s (%s)\n",
 				s, isglobal ? "global" : "local");
@@ -1085,9 +1116,44 @@ int isglobal;
 	pat_compile = s_re_comp;
 	pat_compare = s_re_exec;
 	need_re_anchor = 1;
+#ifdef HAVE_POSIX_REGEX
+    } else if (strncmp(s, "posix", 5) == 0 &&
+			(s[5] == '/' || strlen(s) == 5)) {
+	char *buf = strdup(s);
+	if (!buf) {
+	    return Error(0, 0,
+			"%t\n\tCan't malloc space for copy of <%s>\n", s);
+	}
+	/* Check the options: we allow:
+	 *	posix[/extended][/icase]
+	 */
+	p = strtok(s, "/");
+	if (strcmp(p, "posix") != 0) {
+	    return Error(0, 0, "%t\n\tInternal error: got pat type <%s> when altready checked it's simply `posix'!\n", p);
+	}
+	for (p=strtok(NULL, "/"); p; p=strtok(NULL, "/")) {
+	    if (strcmp(p, "extended") == 0) {
+		p_regcomp_flags(REG_EXTENDED);
+	    } else if (strcmp(p, "icase") == 0) {
+		p_regcomp_flags(REG_ICASE);
+	    } else {
+		return Error(0, 0,
+			    "%t\n\tUnknown posix pattern variant `%s'.\n", p);
+	    }
+	}
+	free(buf);
+	pat_compile = p_compile;
+	pat_compare = p_compare;
+	need_re_anchor = 1;
+#endif
     } else {
+#ifdef HAVE_POSIX_REGEX
+	return Error(0, 0, "%t\n\tInvalid pattern type `%s'.  \
+Valid: \"shell\", \"regex\", and \"posix[/extended][/icase]\"\n", s);
+#else
 	return Error(0, 0, "%t\n\tInvalid pattern type `%s'.  \
 Valid: \"shell\" and \"regex\"\n", s);
+#endif
     }
     return 0;
 }
@@ -1279,6 +1345,8 @@ not compiled with HAVE_SYSLOG defined.\n");
     if (read_syslogcode(s, &syslogcode) == -1)
 	return -1;
     error_priority = syslogcode;
+    if (debug)
+	fprintf(stderr, "\t\t\t=%#x\n", syslogcode);
     return 0;
 }
 
@@ -1302,6 +1370,8 @@ not compiled with HAVE_SYSLOG defined.\n");
     if (read_syslogcode(s, &syslogcode) == -1)
 	return -1;
     globalinfo.log.syslog_success = syslogcode;
+    if (debug)
+	fprintf(stderr, "\t\t\t=%#x\n", syslogcode);
     return 0;
 }
 
