@@ -1,4 +1,4 @@
-static char rcsid[] = "$Id: super.c,v 1.566 2000/09/15 21:49:42 will Exp $";
+static const char rcsid[] = "$Id: super.c,v 1.578 2004/06/18 16:18:20 will Exp $";
 /* The code should compile with either ANSI C or K&R compilers. */
 
 /*
@@ -61,6 +61,7 @@ static char rcsid[] = "$Id: super.c,v 1.566 2000/09/15 21:49:42 will Exp $";
  *		Cmd FullPath [initial args]
  *		...
  *	      Useful for scripts that want a list of what you may execute.
+ *	-S -- force stdin to be used for reading passwords.
  *	-U uid -- act as if the invoking user was user uid; for testing only.
  *		No command will actually be executed.
  *	-V -- print version information, then exit.
@@ -169,6 +170,7 @@ char *shell_compile P__(( char * ));	/* s_re_exec()-style i/f to wildmat() */
 
 char *prog;				/* this program */
 int debug=0;				/* Set by the debug options flag */
+int use_stdin=0;			/* Set by the -S flag */
 int it_came_from_cmdline=0;		/* Set by -F/-T/-U/-G/-M flags */
 int check_syntax=0;			/* Set by the -c options flag */
 int using_user_supertab = 0;		/* !0 means using a user's .supertab */
@@ -213,6 +215,7 @@ char **argv;
     new_core_limit.rlim_max = old_core_limit.rlim_max;
     setrlimit(RLIMIT_CORE, &new_core_limit);
 #endif
+    check_stdio();
     sprintf(superfile_writable, "%s/%s", SUPERDIR, "super.tab");
     sprintf(superfile_init, "%s/%s", SUPERDIR, "super.init");
 
@@ -525,8 +528,11 @@ due to previously-reported problem)\n");
 	/* If here, we failed to exec the prog.  Re-open the logfile we
 	 * closed above and write a message.
 	 */
-	if (*globalinfo.log.filename != '\0')
+	if (*globalinfo.log.filename != '\0') {
+	    int save_errno = errno;
 	    opensuperlog();
+	    errno = save_errno;
+	}
 	(void) Error(1,1, "command `%-.500s': Couldn't exec `%s': ", cmd, path);
     }
     return 0;
@@ -538,8 +544,7 @@ due to previously-reported problem)\n");
  */
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 int
-do_options(argc, argv, givehelp, giveversion, verbosity,
-					printvars, o_file)
+do_options(argc, argv, givehelp, giveversion, verbosity, printvars, o_file)
 int argc;
 char **argv;
 int *givehelp, *giveversion, *verbosity, *printvars;
@@ -547,7 +552,6 @@ char **o_file;
 {
     int iarg;
     char *mhost, *s, *t;
-    short minutes;
 #ifndef HAVE_LOCALTIME
     char daynum;
 #endif
@@ -725,6 +729,7 @@ char **o_file;
 	    case 'D':
 		debug = 2;
 		break;
+	    case 'S': use_stdin = 1; break; break;
 	    case 'V': *giveversion = 1; break;
 	    case '?': *givehelp = 1; *verbosity = HELP_USAGE; break;
 	    case 'h': *givehelp = 1; *verbosity = HELP_USAGE; break;
@@ -857,7 +862,7 @@ int n_builtin;
     for (sp=arglist; *sp; sp++)
 	;
     (void) fprintf(stderr, "\tArgc: %d\n", (int) (sp - arglist));
-    (void) fprintf(stderr, "\tMax len, 1 arg: %d   Max len, all args: %d\n",
+    (void) fprintf(stderr, "\tMax len, 1 arg: %ld   Max len, all args: %ld\n",
 				localinfo.maxlen1arg, localinfo.maxlenargs);
     for (sp=arglist; *sp; sp++) {
 	iarg = sp - arglist;
@@ -937,8 +942,8 @@ int n_builtin;
     if (localinfo.fdlist)
 	(void) fprintf(stderr, ",%s", localinfo.fdlist);
     (void) fprintf(stderr, "\n\n\tID's:\treal effective\n");
-    (void) fprintf(stderr, "\tuid\t%d\t%d\n", userinfo.orig_uid, geteuid());
-    (void) fprintf(stderr, "\tgid\t%d\t%d\n", userinfo.orig_gid, getegid());
+    (void) fprintf(stderr, "\tuid\t%d\t%d\n", getuid(), geteuid());
+    (void) fprintf(stderr, "\tgid\t%d\t%d\n", getgid(), getegid());
 #ifdef HAVE_GETGROUPS
     {
 	GETGROUPS_T groups[NGROUPS_MAX];
@@ -988,11 +993,10 @@ char *file_or_user;	/* either file from `-o file' or username */
 int is_file;		/* !0 means it's filename, not username */
 char *cmd;		/* for error reporting */
 {
-    char *s, *user;
+    char  *user;
     struct passwd *pass;
-    uid_t uid;
+    int i, j;
     struct stat sto, stu;
-    static char no_cmd[] = "";
 
     if (!cmd)
 	cmd = "";
@@ -1043,15 +1047,19 @@ but it isn't the expected file `%s'\n",
 Command <%s:%-.500s>; uid=%d, euid=%d: ",
 	    user, pass->pw_gid, user, cmd, getuid(), geteuid());
 
-    if (setgid(pass->pw_gid) == -1)
-	Error(1, 1,
-	    "Can't set gid to that of user %s (from command <%s:%-.500s>): ",
-	    user, user, cmd);
+    if ((i=setgid(pass->pw_gid)) == -1) {
+	Error(1, 1, "Can't set gid to %d\n", pass->pw_gid);
+    } else if ((j=getgid()) != pass->pw_gid) {
+	Error(1, 1, "setgid(gid=%d) returned %d, but getgid() returned %d!",
+	    pass->pw_gid, i, j);
+    }
 
-    if (setuid(pass->pw_uid) == -1)
-	Error(1, 1,
-	    "Can't set uid to user %s (from command <%s:%-.500s>): ",
-	    user, user, cmd);
+    if ((i=setuid(pass->pw_uid)) == -1) {
+	Error(1, 1, "Can't set uid to %d\n", pass->pw_uid);
+    } else if ((j=getuid()) != pass->pw_uid) {
+	Error(1, 1, "setuid(uid=%d) returned %d, but getuid() returned %d!",
+	    pass->pw_uid, i, j);
+    }
 }
 
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
@@ -1119,7 +1127,7 @@ char **args;
 	logbuf[loglen-2] = '\n';
 	logbuf[loglen-1] = '\0';
     } else {
-	strcpy(logbuf, cmd);
+	sprintf(logbuf, "%s\n", cmd);
     }
 
     /* Log the message using Error(), but
@@ -1132,7 +1140,7 @@ char **args;
 	    (localinfo.mail_success == -1 && globalinfo.mail_success==0))
 	error_command = NULL;
     error_stderr = 0;
-    Error(0, 0, logbuf);
+    Error(0, 0, "%s", logbuf);
     error_stderr = e;
     error_command = ec;
 } 
@@ -1181,7 +1189,7 @@ int *n_builtin;		/* returned w/ number of args in path_plus */
     /* Check that each user-entered argument matches its pattern, if given */
     for (iarg = 1; iarg <= nargs; iarg++) {
 	s = StrEltGetPtr(&localinfo.argpats, iarg);
-	if (s && match_pattern(0, 0, argv[iarg], s) != 1)
+	if (s && match_pattern(0, 1, argv[iarg], s) != 1)
 	    (void) Error(0, 2,
 		"Your argument #%d <%-.500s> must match pattern <%s>\n",
 		iarg, argv[iarg], s);
@@ -1323,7 +1331,8 @@ char *cmd;		/* name of command being started */
     for (i=0; i<NSIG; i++)
        (void) signal(i, SIG_DFL);
 
-    s = *localinfo.user ? localinfo.user :
+    s = *localinfo.euser ? localinfo.euser :
+	    *localinfo.user ? localinfo.user :
 		*localinfo.u_g ? localinfo.u_g : userinfo.caller.pw_name;
     (void) sprintf(OrigUser, "ORIG_USER=%s", userinfo.caller.pw_name);
     (void) sprintf(Caller, "CALLER=%s", userinfo.caller.pw_name);
@@ -1524,8 +1533,6 @@ init_globalinfo()
     for (i=0; i<=MAXSETENV; i++)
 	globalinfo.setenv[i]=NULL;
 
-    error_priority = LOG_ERR;
-    error_facility = LOG_USER;
 }
 
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
@@ -1567,7 +1574,6 @@ int
 init_userinfo()
 {
     struct passwd *usrpw;
-    char *s;
 
     userinfo.ourtime.start = time(NULL);
 #ifdef HAVE_LOCALTIME
@@ -1675,6 +1681,31 @@ set_chdir()
     return 0;
 }
 
+/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
+/* Make sure that stdin, stdout, stderr are all open; die 
+ * if we cannot do so.
+ */
+/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
+void
+check_stdio()
+{
+    struct stat stat;
+    if (fstat(0, &stat) == -1 && !freopen("/dev/null", "r", stdin)) {
+	exit(1);
+    }
+    if (fstat(1, &stat) == -1 && !freopen("/dev/null", "w", stdout)) {
+	exit(1);
+    }
+    if (fstat(2, &stat) == -1 && !freopen("/dev/null", "w", stderr)) {
+	exit(1);
+    }
+#ifdef HAVE_FILENO
+    if (fileno(stdin) != 0) { exit(1); }
+    if (fileno(stdout) != 1) { exit(1); }
+    if (fileno(stderr) != 2) { exit(1); }
+#endif
+}
+
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 /* Store/set/recall niceness */
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
