@@ -160,6 +160,7 @@ void *memset P__(( void *s, int c, size_t n ));
     extern char *prog;			/* this program */
     extern int debug;			/* debug level; 0=off */
     extern int check_syntax;		/* just check syntax of superfile */
+    extern int use_stdin;		/* force stdin for passwds */
     extern char *superfile;		/* The actual superfile to be opened. */
     extern char superfile_init[];	/* The super.init file. */
     extern int it_came_from_cmdline;	/* Set by -F/-T/-U/-G flags */
@@ -391,12 +392,14 @@ struct userInfo {
     gid_t orig_gid;			/* gid at prog start, from getgid() */
     uid_t new_uid;			/* new uid, from uid=xxx or u+g=xxx */
     gid_t new_gid;			/* new gid, from gid=xxx or u+g=xxx */
+    uid_t new_euid;			/* new euid, from uid=xxx or euid=xxx */
+    gid_t new_egid;			/* new egid, from gid=xxx or egid=xxx */
 
     OurTime ourtime;			/* when we started, etc */
 
-    char encr[256];			/* encrypted password; length is */
+    char encr[1000];			/* encrypted password; length is */
 					/* pretty large by current standards */
-    char salt[256];			/* salt from password */
+    char salt[1000];			/* salt from password */
 };
 typedef struct userInfo UserInfo;
 
@@ -413,8 +416,10 @@ struct localInfo {
     char *chdir_path;	/* Change to this dir before exec'ing; null if none */
     char *argv0;	/* value of argv0=xxx option */
 
-    char user[32];	/* value of uid=xxx option */
-    char group[32];	/* value of gid=xxx option */
+    char user[32];	/* value of uid=xxx options */
+    char euser[32];	/* value of euid=xxx options */
+    char group[32];	/* value of group=xxx options */
+    char egroup[32];	/* value of egroup=xxx options */
     char u_g[32];	/* value of u+g=xxx option */
     char owner[32];	/* value of owner=xxx option */
     uid_t file_uid;	/* uid of the matched FullPath */
@@ -468,6 +473,7 @@ int	add_uname_variables P__(( void ));
 int	add_variable P__(( char *varname, char *vardefn ));
 void	anchor P__(( char *in, char *out ));
 char*	approve P__((char *usrcmd, int givehelp, int verbose));
+char*	auth_name P__(( void ));
 char ** blkdup P__((char **blk));
 int	blkfree P__((char **av0));
 char ** Blkdup P__((char *str, char **blk));
@@ -475,15 +481,19 @@ int	Blkfree P__((char *str, char **av0));
 int	blkprint P__((char **));
 int	build_cmd_file P__((char *, int , char *, char **));
 char**	buttonup P__((char *));
+int	canonicalize_hostname P__(( char *buf, int len));
 int	check_arglistlen P__(( char **argv ));
 int	check_auth P__(( char *cmd ));
 int	check_owner P__(( void ));
+void	check_stdio P__((void));
 int	check_value P__(( char *value, char *pat ));
 int	check_var_value P__(( void ));
+int	checkarg P__(( char *str ));
 int	checkenv P__(( char *name,  char *value,  char *pat ));
 char*	clean_buf P__(( char *buf, char *outbuf ));
 void	close_writer P__(( void ));
 int	colon_define P__(( char *command ));
+int	colon_die P__(( char *command ));
 int	colon_getenv P__(( char *command ));
 int	colon_global P__(( char *command ));
 int	colon_include P__(( char *command, int allow_missing ));
@@ -496,7 +506,7 @@ int	daynum P__(( int unixtime ));
 void	debug_print P__((char *path, char **arglist, char **envp, int n_bltin));
 void	debug_hello P__((void));
 int	do_options P__((int argc, char **argv, int *help, int *vers,
-			int *verbose, int *printvars, char **owned_by_file));
+		    int *verbose, int *printvars, char **owned_by_file));
 char*	do_variables P__(( char *str));
 char*	docrypt P__((char *key, char *salt));
 char*	ends P__(( char *s1, char *s2 ));
@@ -554,6 +564,9 @@ void	opensuperlog P__(( void ));
 int	option_clear_settings P__((char *word, char *s, int isglobal));
 int	option_global_reset_settings P__((void));
 int	option_local_clear_settings P__((void));
+void	p_regcomp_flags P__(( int ));
+char*	p_compile P__(( char *));
+int	p_compare P__(( char *));
 int	parseline P__((int givehelp, int checksyntax, char *buf, char *usrcmd));
 void	print_variable P__(( int indx, char *key, char *data ));
 void	printhelp P__(( int verbose ));
@@ -563,6 +576,7 @@ int	process_logfile_opt P__(( void ));
 int	rcl_nice_incr P__((void));
 int	rcl_umask P__((void));
 void	re_anchor P__(( char *in, char *out));
+int	s_getpass P__((char *prompt, int use_stdin, char *buf, int buflen));
 char*	s_re_comp P__(( char *));
 int	s_re_exec P__(( char *));
 void	readtime_init P__(( void ));
@@ -591,3 +605,4 @@ void	strtolower P__(( char *string ));
 char*	strqtokS P__(( char *, char *, char *, char *, unsigned int ));
 int	read_syslogcode P__(( char *str, int *code ));
 void	user_supertab P__(( char *file_or_user, int is_file, char *cmd ));
+int	wildmat P__(( char *text, char *p ));
