@@ -1,4 +1,4 @@
-static char rcsid[] = "$Id: utils.c,v 1.118 2000/08/20 04:55:13 will Exp $";
+static const char rcsid[] = "$Id: utils.c,v 1.126 2004/04/30 17:00:58 will Exp $";
 #include "super.h"
 
 /* Variables table flag -- !0 means has been created */
@@ -59,7 +59,6 @@ char *buf;
      */
  
     struct passwd *pass;
-    char *p;
  
     buf[0] = '\0';
     if (user != NULL && *user != '\0') {
@@ -79,7 +78,7 @@ char *buf;
 }
  
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
-/* Convert string to syslog priority code
+/* Convert string to syslog priority code */
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 int
 read_syslogcode(s, code)
@@ -87,27 +86,46 @@ char *s;
 int *code;
 {
     /*
-     * Takes a string like "LOG_AUTH | LOG_LOCAL1", and returns the
+     * Takes a string like "LOG_WARNING | LOG_AUTH", and returns the
      * corresponding syslog numeric value in *code.
+     * The leading "LOG_" is optional, as is the case, and the separator
+     * can be any combination of space, dot, and |.  Thus the following
+     * are equivalent:
+     * 		LOG_WARNING | LOG_AUTH
+     * 		warning.auth
      * Returns:
      *	0 on success;
      *	-1 on error, with error message to Error().
      *
-     * In s, the valid code separators are: whitespace and/or "|".
+     * In s, the valid code separators are: whitespace, '.'  and/or "|".
      *
      */
-    char *word;
-    char *sep = " \t|";
+    char *word, *xword;
+    char *sep = " \t|.";
 
-    char *new_s;
+    char *p, *new_s;
 
     if (!(new_s = strdup(s))) {
 	return Error(0, 0,
 	    "Failed to malloc space for copy of s=<%s>\n", s);
     }
+    for (p = new_s; *p; p++) {
+	if (islower(*p)) {
+	    *p = toupper(*p);
+	}
+    }
 
     *code = 0;
     for (word = strtok(new_s, sep); word; word = strtok(NULL, sep)) {
+	if (!word) {
+	    continue;	/* ignore empty field */
+	}
+	if (strncmp(word, "LOG_", 4) == 0) {
+	    xword = word + 4;		/* ignore leading LOG_ */
+	} else {
+	    xword = word;		/* there is no leading LOG_ */
+	}
+
 	if (0) {
 	    /* placeholder */
 	/*
@@ -116,79 +134,79 @@ int *code;
 	 * **********
 	 */
 #ifdef LOG_AUTH
-	} else if (strcmp(word, "LOG_AUTH") == 0) {
+	} else if (strcmp(xword, "AUTH") == 0) {
 	    *code |= LOG_AUTH;
 #endif
 #ifdef LOG_AUTHPRIV
-	} else if (strcmp(word, "LOG_AUTHPRIV") == 0) {
+	} else if (strcmp(xword, "AUTHPRIV") == 0) {
 	    *code |= LOG_AUTHPRIV;
 #endif
 #ifdef LOG_CRON
-	} else if (strcmp(word, "LOG_CRON") == 0) {
+	} else if (strcmp(xword, "CRON") == 0) {
 	    *code |= LOG_CRON;
 #endif
 #ifdef LOG_DAEMON
-	} else if (strcmp(word, "LOG_DAEMON") == 0) {
+	} else if (strcmp(xword, "DAEMON") == 0) {
 	    *code |= LOG_DAEMON;
 #endif
 #ifdef LOG_KERN
-	} else if (strcmp(word, "LOG_KERN") == 0) {
+	} else if (strcmp(xword, "KERN") == 0) {
 	    *code |= LOG_KERN;
 #endif
 #ifdef LOG_LOCAL0
-	} else if (strcmp(word, "LOG_LOCAL0") == 0) {
+	} else if (strcmp(xword, "LOCAL0") == 0) {
 	    *code |= LOG_LOCAL0;
 #endif
 #ifdef LOG_LOCAL1
-	} else if (strcmp(word, "LOG_LOCAL1") == 0) {
+	} else if (strcmp(xword, "LOCAL1") == 0) {
 	    *code |= LOG_LOCAL1;
 #endif
 #ifdef LOG_LOCAL2
-	} else if (strcmp(word, "LOG_LOCAL2") == 0) {
+	} else if (strcmp(xword, "LOCAL2") == 0) {
 	    *code |= LOG_LOCAL2;
 #endif
 #ifdef LOG_LOCAL3
-	} else if (strcmp(word, "LOG_LOCAL3") == 0) {
+	} else if (strcmp(xword, "LOCAL3") == 0) {
 	    *code |= LOG_LOCAL3;
 #endif
 #ifdef LOG_LOCAL4
-	} else if (strcmp(word, "LOG_LOCAL4") == 0) {
+	} else if (strcmp(xword, "LOCAL4") == 0) {
 	    *code |= LOG_LOCAL4;
 #endif
 #ifdef LOG_LOCAL5
-	} else if (strcmp(word, "LOG_LOCAL5") == 0) {
+	} else if (strcmp(xword, "LOCAL5") == 0) {
 	    *code |= LOG_LOCAL5;
 #endif
 #ifdef LOG_LOCAL6
-	} else if (strcmp(word, "LOG_LOCAL6") == 0) {
+	} else if (strcmp(xword, "LOCAL6") == 0) {
 	    *code |= LOG_LOCAL6;
 #endif
 #ifdef LOG_LOCAL7
-	} else if (strcmp(word, "LOG_LOCAL7") == 0) {
+	} else if (strcmp(xword, "LOCAL7") == 0) {
 	    *code |= LOG_LOCAL7;
 #endif
 #ifdef LOG_LPR
-	} else if (strcmp(word, "LOG_LPR") == 0) {
+	} else if (strcmp(xword, "LPR") == 0) {
 	    *code |= LOG_LPR;
 #endif
 #ifdef LOG_MAIL
-	} else if (strcmp(word, "LOG_MAIL") == 0) {
+	} else if (strcmp(xword, "MAIL") == 0) {
 	    *code |= LOG_MAIL;
 #endif
 #ifdef LOG_NEWS
-	} else if (strcmp(word, "LOG_NEWS") == 0) {
+	} else if (strcmp(xword, "NEWS") == 0) {
 	    *code |= LOG_NEWS;
 #endif
 #ifdef LOG_RFS
-	} else if (strcmp(word, "LOG_NEWS") == 0) {
+	} else if (strcmp(xword, "RFS") == 0) {
 	    *code |= LOG_RFS;
 #endif
 #ifdef LOG_USER
-	} else if (strcmp(word, "LOG_USER") == 0) {
+	} else if (strcmp(xword, "USER") == 0) {
 	    *code |= LOG_USER;
 #endif
 #ifdef LOG_UUCP
-	} else if (strcmp(word, "LOG_UUCP") == 0) {
+	} else if (strcmp(xword, "UUCP") == 0) {
 	    *code |= LOG_UUCP;
 #endif
 	/*
@@ -197,35 +215,35 @@ int *code;
 	 * ******
 	 */
 #ifdef LOG_ALERT
-	} else if (strcmp(word, "LOG_ALERT") == 0) {
+	} else if (strcmp(xword, "ALERT") == 0) {
 	    *code |= LOG_ALERT;
 #endif
 #ifdef LOG_CRIT
-	} else if (strcmp(word, "LOG_CRIT") == 0) {
+	} else if (strcmp(xword, "CRIT") == 0) {
 	    *code |= LOG_CRIT;
 #endif
 #ifdef LOG_DEBUG
-	} else if (strcmp(word, "LOG_DEBUG") == 0) {
+	} else if (strcmp(xword, "DEBUG") == 0) {
 	    *code |= LOG_DEBUG;
 #endif
 #ifdef LOG_EMERG
-	} else if (strcmp(word, "LOG_EMERG") == 0) {
+	} else if (strcmp(xword, "EMERG") == 0) {
 	    *code |= LOG_EMERG;
 #endif
 #ifdef LOG_ERR
-	} else if (strcmp(word, "LOG_ERR") == 0) {
+	} else if (strcmp(xword, "ERR") == 0) {
 	    *code |= LOG_ERR;
 #endif
 #ifdef LOG_INFO
-	} else if (strcmp(word, "LOG_INFO") == 0) {
+	} else if (strcmp(xword, "INFO") == 0) {
 	    *code |= LOG_INFO;
 #endif
 #ifdef LOG_NOTICE
-	} else if (strcmp(word, "LOG_NOTICE") == 0) {
+	} else if (strcmp(xword, "NOTICE") == 0) {
 	    *code |= LOG_NOTICE;
 #endif
 #ifdef LOG_WARNING
-	} else if (strcmp(word, "LOG_WARNING") == 0) {
+	} else if (strcmp(xword, "WARNING") == 0) {
 	    *code |= LOG_WARNING;
 #endif
 	} else {
@@ -962,6 +980,8 @@ pid_t *pid_p;	/* filled with pid of created logger */
 	}
 	if (user && *user != '\0') {
 	    stringcopy(localinfo.user, user, sizeof(localinfo.user));
+	    *localinfo.euser = '\0';
+	    *localinfo.egroup = '\0';
 	    *localinfo.group = '\0';
 	    *localinfo.u_g = '\0';
 	    if (set_u_g() == -1) {
@@ -1177,7 +1197,13 @@ int
 add_sysinfo_variables()
 {
     char buf[500];
-    add_variable("SI_ARCHITECTURE", "");
+
+    /* silly gimmick: define a variable using empty buf[] instead of "",
+     * so that gcc -Wall doesn't complain if buf isn't used in at least
+     * one of the ifdef'd regions, below.
+     */
+    buf[0] = '\0';
+    add_variable("SI_ARCHITECTURE", buf);
 #ifdef HAVE_SYSINFO
 #ifdef SI_ARCHITECTURE
     if (sysinfo(SI_ARCHITECTURE, buf, sizeof(buf)) != -1) {
@@ -1274,6 +1300,7 @@ add_sysinfo_variables()
     }
 #endif
 #endif
+    return 0;
 }
 
 #ifdef HAVE_UNAME
@@ -1300,6 +1327,7 @@ add_uname_variables()
 	    add_variable("UNAME_MACHINE", ""))
 		return -1;
     }
+    return 0;
 }
 #endif
 
